//
// dccp_endpoint.hpp
// ~~~~~~~~~~~~~~~~~~
//
// Copyright (c) 2003-2008 Christopher M. Kohlhoff (chris at kohlhoff dot com)
// Copyright (c) 2009 Alex Dubov (oakad at yahoo dot com)
//
// Distributed under the Boost Software License, Version 1.0. (See accompanying
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
//

#ifndef BOOST_ASIO_IP_DCCP_ENDPOINT_HPP
#define BOOST_ASIO_IP_DCCP_ENDPOINT_HPP

#if defined(_MSC_VER) && (_MSC_VER >= 1200)
# pragma once
#endif // defined(_MSC_VER) && (_MSC_VER >= 1200)

#include <boost/asio/detail/push_options.hpp>

#include <boost/asio/error.hpp>
#include <boost/asio/ip/basic_endpoint.hpp>

namespace boost {
namespace asio {
namespace ip {

class dccp;

/// Describes an endpoint for a version-independent IP socket.
/**
 * The boost::asio::ip::dccp_endpoint class template describes an endpoint that
 * may be associated with a particular dccp socket.
 *
 * @par Thread Safety
 * @e Distinct @e objects: Safe.@n
 * @e Shared @e objects: Unsafe.
 *
 * @par Concepts:
 * Endpoint.
 */
class dccp_endpoint
  : public basic_endpoint<dccp>
{
public:
  dccp_endpoint()
  {
  }

  dccp_endpoint(const dccp& protocol, unsigned short port_num,
      const std::vector<unsigned int>& sc = std::vector<unsigned int>())
    : basic_endpoint<dccp>(protocol, port_num),
      service_codes_(sc)
  {
  }

  dccp_endpoint(const dccp& protocol, unsigned short port_num,
                unsigned int sc)
    : basic_endpoint<dccp>(protocol, port_num),
      service_codes_(1, sc)
  {
  }

  dccp_endpoint(const dccp_endpoint& other)
    : basic_endpoint<dccp>(other),
      service_codes_(other.service_codes_)
  {
  }

  dccp_endpoint(const dccp_endpoint& other, const std::vector<unsigned int>& sc)
    : basic_endpoint<dccp>(other),
      service_codes_(sc)
  {
  }

  dccp_endpoint(const dccp_endpoint& other, unsigned int sc)
    : basic_endpoint<dccp>(other),
      service_codes_(1, sc)
  {
  }

  dccp_endpoint& operator=(const dccp_endpoint& other)
  {
    basic_endpoint<dccp>::operator=(other);
    service_codes_ = other.service_codes_;
    return *this;
  }

  const std::vector<unsigned int>& service_codes() const
  {
    return service_codes_;
  }

  void service_codes(const std::vector<unsigned int>& sc)
  {
    service_codes_ = sc;
  }

  void service_codes(const unsigned int sc)
  {
    service_codes_.clear();
    service_codes_.push_back(sc);
  }

 /// Compare two endpoints for equality.
  friend bool operator==(const dccp_endpoint& e1,
      const dccp_endpoint& e2)
  {
    return dynamic_cast<const basic_endpoint<dccp>&>(e1)
           == dynamic_cast<const basic_endpoint<dccp>&>(e2)
           && e1.service_codes_ == e2.service_codes_;
  }

  /// Compare two endpoints for inequality.
  friend bool operator!=(const dccp_endpoint& e1,
      const dccp_endpoint& e2)
  {
    return dynamic_cast<const basic_endpoint<dccp>&>(e1)
           != dynamic_cast<const basic_endpoint<dccp>&>(e2)
           || e1.service_codes_ != e2.service_codes_;
  }

private:
  std::vector<unsigned int> service_codes_;
};

} // namespace ip
} // namespace asio
} // namespace boost

#include <boost/asio/detail/pop_options.hpp>

#endif // BOOST_ASIO_IP_DCCP_ENDPOINT_HPP
