//
// dccp_socket_acceptor.hpp
// ~~~~~~~~~~~~~~~~~~~~~~~~~
//
// Copyright (c) 2003-2008 Christopher M. Kohlhoff (chris at kohlhoff dot com)
// Copyright (c) 2009 Alex Dubov (oakad at yahoo dot com)
//
// Distributed under the Boost Software License, Version 1.0. (See accompanying
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
//

#ifndef BOOST_ASIO_DCCP_SOCKET_ACCEPTOR_HPP
#define BOOST_ASIO_DCCP_SOCKET_ACCEPTOR_HPP

#if defined(_MSC_VER) && (_MSC_VER >= 1200)
# pragma once
#endif // defined(_MSC_VER) && (_MSC_VER >= 1200)

#include <boost/asio/detail/push_options.hpp>

#include <boost/asio/basic_io_object.hpp>
#include <boost/asio/error.hpp>
#include <boost/asio/socket_acceptor_service.hpp>
#include <boost/asio/socket_base.hpp>
#include <boost/asio/detail/throw_error.hpp>

#include "dccp_option.hpp"

namespace boost {
namespace asio {
namespace ip {
class dccp;
}

/// Provides the ability to accept new connections.
/**
 * The basic_socket_acceptor class template is used for accepting new socket
 * connections.
 *
 * @par Thread Safety
 * @e Distinct @e objects: Safe.@n
 * @e Shared @e objects: Unsafe.
 *
 * @par Example
 * Opening a socket acceptor with the SO_REUSEADDR option enabled:
 * @code
 * boost::asio::ip::dccp::acceptor acceptor(io_service);
 * boost::asio::ip::dccp::endpoint endpoint(boost::asio::ip::dccp::v4(), port);
 * acceptor.open(endpoint.protocol());
 * acceptor.set_option(boost::asio::ip::dccp::acceptor::reuse_address(true));
 * acceptor.bind(endpoint);
 * acceptor.listen();
 * @endcode
 */
template <typename Protocol = ip::dccp,
    typename SocketAcceptorService = socket_acceptor_service<Protocol> >
class dccp_socket_acceptor
  : public basic_socket_acceptor<Protocol, SocketAcceptorService>
{
public:
  /// The native representation of an acceptor.
  typedef typename SocketAcceptorService::native_type native_type;

  /// The protocol type.
  typedef Protocol protocol_type;

  /// The endpoint type.
  typedef typename Protocol::endpoint endpoint_type;

  /// Construct an acceptor without opening it.
  /**
   * This constructor creates an acceptor without opening it to listen for new
   * connections. The open() function must be called before the acceptor can
   * accept new socket connections.
   *
   * @param io_service The io_service object that the acceptor will use to
   * dispatch handlers for any asynchronous operations performed on the
   * acceptor.
   */
  explicit dccp_socket_acceptor(boost::asio::io_service& io_service)
    : basic_socket_acceptor<Protocol, SocketAcceptorService>(io_service)
  {
  }

  /// Construct an open acceptor.
  /**
   * This constructor creates an acceptor and automatically opens it.
   *
   * @param io_service The io_service object that the acceptor will use to
   * dispatch handlers for any asynchronous operations performed on the
   * acceptor.
   *
   * @param protocol An object specifying protocol parameters to be used.
   *
   * @throws boost::system::system_error Thrown on failure.
   */
  dccp_socket_acceptor(boost::asio::io_service& io_service,
      const protocol_type& protocol)
    : basic_socket_acceptor<Protocol, SocketAcceptorService>(io_service, protocol)
  {
  }

  /// Construct an acceptor opened on the given endpoint.
  /**
   * This constructor creates an acceptor and automatically opens it to listen
   * for new connections on the specified endpoint.
   *
   * @param io_service The io_service object that the acceptor will use to
   * dispatch handlers for any asynchronous operations performed on the
   * acceptor.
   *
   * @param endpoint An endpoint on the local machine on which the acceptor
   * will listen for new connections.
   *
   * @param reuse_addr Whether the constructor should set the socket option
   * socket_base::reuse_address.
   *
   * @throws boost::system::system_error Thrown on failure.
   *
   * @note This constructor is equivalent to the following code:
   * @code
   * basic_socket_acceptor<Protocol> acceptor(io_service);
   * acceptor.open(endpoint.protocol());
   * if (reuse_addr)
   *   acceptor.set_option(socket_base::reuse_address(true));
   * acceptor.bind(endpoint);
   * acceptor.listen(listen_backlog);
   * @endcode
   */
  dccp_socket_acceptor(boost::asio::io_service& io_service,
      const endpoint_type& endpoint, bool reuse_addr = true)
    : basic_socket_acceptor<Protocol, SocketAcceptorService>(
         io_service, endpoint.protocol())
  {
    boost::system::error_code ec;
    if (reuse_addr)
    {
      this->service.set_option(this->implementation,
          socket_base::reuse_address(true), ec);
      boost::asio::detail::throw_error(ec);
    }

    if (!endpoint.service_codes().empty())
    {
      dccp_option::service option(endpoint.service_codes());
      this->service.set_option(this->implementation, option, ec);
      boost::asio::detail::throw_error(ec);
    }

    this->service.bind(this->implementation, endpoint, ec);
    boost::asio::detail::throw_error(ec);
    this->service.listen(this->implementation,
        socket_base::max_connections, ec);
    boost::asio::detail::throw_error(ec);
  }

  /// Construct a basic_socket_acceptor on an existing native acceptor.
  /**
   * This constructor creates an acceptor object to hold an existing native
   * acceptor.
   *
   * @param io_service The io_service object that the acceptor will use to
   * dispatch handlers for any asynchronous operations performed on the
   * acceptor.
   *
   * @param protocol An object specifying protocol parameters to be used.
   *
   * @param native_acceptor A native acceptor.
   *
   * @throws boost::system::system_error Thrown on failure.
   */
  dccp_socket_acceptor(boost::asio::io_service& io_service,
      const protocol_type& protocol, const native_type& native_acceptor)
    : basic_socket_acceptor<Protocol, SocketAcceptorService>(
          io_service, protocol, native_acceptor)
  {
  }

  /// Determine whether the acceptor is open.
  bool is_open() const
  {
    return this->service.is_open(this->implementation);
  }

  /// Bind the acceptor to the given local endpoint.
  /**
   * This function binds the socket acceptor to the specified endpoint on the
   * local machine.
   *
   * @param endpoint An endpoint on the local machine to which the socket
   * acceptor will be bound.
   *
   * @throws boost::system::system_error Thrown on failure.
   *
   * @par Example
   * @code
   * boost::asio::ip::dccp::acceptor acceptor(io_service);
   * acceptor.open(boost::asio::ip::dccp::v4());
   * acceptor.bind(boost::asio::ip::dccp::endpoint(12345));
   * @endcode
   */
  void bind(const endpoint_type& endpoint)
  {
    boost::system::error_code ec;

    if (!endpoint.service_codes().empty())
    {
      dccp_option::service option(endpoint.service_codes());
      this->service.set_option(option, ec);
      boost::asio::detail::throw_error(ec);
    }

    this->service.bind(this->implementation, endpoint, ec);
    boost::asio::detail::throw_error(ec);
  }

  /// Bind the acceptor to the given local endpoint.
  /**
   * This function binds the socket acceptor to the specified endpoint on the
   * local machine.
   *
   * @param endpoint An endpoint on the local machine to which the socket
   * acceptor will be bound.
   *
   * @param ec Set to indicate what error occurred, if any.
   *
   * @par Example
   * @code
   * boost::asio::ip::dccp::acceptor acceptor(io_service);
   * acceptor.open(boost::asio::ip::dccp::v4());
   * boost::system::error_code ec;
   * acceptor.bind(boost::asio::ip::dccp::endpoint(12345), ec);
   * if (ec)
   * {
   *   // An error occurred.
   * }
   * @endcode
   */
   boost::system::error_code bind(const endpoint_type& endpoint,
      boost::system::error_code& ec)
  {
    if (!endpoint.service_codes().empty())
    {
      boost::system::error_code ec;
      dccp_option::service option(endpoint.service_codes());
      if (this->service.set_option(option, ec))
        return ec;
    }

    return this->service.bind(this->implementation, endpoint, ec);
  }
};

} // namespace asio
} // namespace boost

#include <boost/asio/detail/pop_options.hpp>

#endif // BOOST_ASIO_DCCP_SOCKET_ACCEPTOR_HPP
